summaryrefslogtreecommitdiffstats
path: root/Lib/SocketServer.py
diff options
context:
space:
mode:
authorGeorg Brandl <georg@python.org>2008-01-19 20:08:23 (GMT)
committerGeorg Brandl <georg@python.org>2008-01-19 20:08:23 (GMT)
commitfceab5a385f121029bfa7e855f7fd1aca1eb8c3f (patch)
tree91d552ec2d068287efc8b8f3769cd6865c3c0620 /Lib/SocketServer.py
parent2336bddd5dba559db950e8b5fa73257fc62d1fc3 (diff)
downloadcpython-fceab5a385f121029bfa7e855f7fd1aca1eb8c3f.zip
cpython-fceab5a385f121029bfa7e855f7fd1aca1eb8c3f.tar.gz
cpython-fceab5a385f121029bfa7e855f7fd1aca1eb8c3f.tar.bz2
Merged revisions 60080-60089,60091-60093 via svnmerge from
svn+ssh://pythondev@svn.python.org/python/trunk ........ r60080 | andrew.kuchling | 2008-01-19 17:26:13 +0100 (Sat, 19 Jan 2008) | 2 lines Patch #742598 from Michael Pomraning: add .timeout attribute to SocketServer that will call .handle_timeout() method when no requests are received within the timeout period. ........ r60081 | andrew.kuchling | 2008-01-19 17:34:09 +0100 (Sat, 19 Jan 2008) | 1 line Add item ........ r60082 | christian.heimes | 2008-01-19 17:39:27 +0100 (Sat, 19 Jan 2008) | 2 lines Disabled test_xmlrpc:test_404. It's causing lots of false alarms. I also disabled a test in test_ssl which requires network access to svn.python.org. This fixes a bug Skip has reported a while ago. ........ r60083 | georg.brandl | 2008-01-19 18:38:53 +0100 (Sat, 19 Jan 2008) | 2 lines Clarify thread.join() docs. #1873. ........ r60084 | georg.brandl | 2008-01-19 19:02:46 +0100 (Sat, 19 Jan 2008) | 2 lines #1782: don't leak in error case in PyModule_AddXxxConstant. Patch by Hrvoje Nik?\197?\161i?\196?\135. ........ r60085 | andrew.kuchling | 2008-01-19 19:08:52 +0100 (Sat, 19 Jan 2008) | 1 line Sort two names into position ........ r60086 | andrew.kuchling | 2008-01-19 19:18:41 +0100 (Sat, 19 Jan 2008) | 2 lines Patch #976880: add mmap .rfind() method, and 'end' paramter to .find(). Contributed by John Lenton. ........ r60087 | facundo.batista | 2008-01-19 19:38:19 +0100 (Sat, 19 Jan 2008) | 5 lines Fix #1693149. Now you can pass several modules separated by coma to trace.py in the same --ignore-module option. Thanks Raghuram Devarakonda. ........ r60088 | facundo.batista | 2008-01-19 19:45:46 +0100 (Sat, 19 Jan 2008) | 3 lines Comment in NEWS regarding the change in trace.py. ........ r60089 | skip.montanaro | 2008-01-19 19:47:24 +0100 (Sat, 19 Jan 2008) | 2 lines missing from r60088 checkin. ........ r60091 | andrew.kuchling | 2008-01-19 20:14:05 +0100 (Sat, 19 Jan 2008) | 1 line Add item ........ r60092 | georg.brandl | 2008-01-19 20:27:05 +0100 (Sat, 19 Jan 2008) | 4 lines Fix #1679: "0x" was taken as a valid integer literal. Fixes the tokenizer, tokenize.py and int() to reject this. Patches by Malte Helmert. ........ r60093 | georg.brandl | 2008-01-19 20:48:19 +0100 (Sat, 19 Jan 2008) | 3 lines Fix #1146: TextWrap vs words 1-character shorter than the width. Patch by Quentin Gallet-Gilles. ........
Diffstat (limited to 'Lib/SocketServer.py')
-rw-r--r--Lib/SocketServer.py45
1 files changed, 41 insertions, 4 deletions
diff --git a/Lib/SocketServer.py b/Lib/SocketServer.py
index 994a3c6..f62b7df 100644
--- a/Lib/SocketServer.py
+++ b/Lib/SocketServer.py
@@ -158,6 +158,7 @@ class BaseServer:
- server_bind()
- server_activate()
- get_request() -> request, client_address
+ - handle_timeout()
- verify_request(request, client_address)
- server_close()
- process_request(request, client_address)
@@ -171,6 +172,7 @@ class BaseServer:
Class variables that may be overridden by derived classes or
instances:
+ - timeout
- address_family
- socket_type
- allow_reuse_address
@@ -182,6 +184,8 @@ class BaseServer:
"""
+ timeout = None
+
def __init__(self, server_address, RequestHandlerClass):
"""Constructor. May be extended, do not override."""
self.server_address = server_address
@@ -204,8 +208,9 @@ class BaseServer:
# finishing a request is fairly arbitrary. Remember:
#
# - handle_request() is the top-level call. It calls
# get_request(), verify_request() and process_request()
# - get_request() is different for stream or datagram sockets
# - process_request() is the place that may fork a new process
# or create a new thread to finish the request
# - finish_request() instantiates the request handler class;
@@ -214,7 +219,7 @@ class BaseServer:
def handle_request(self):
"""Handle one request, possibly blocking."""
try:
- request, client_address = self.get_request()
+ request, client_address = self.await_request()
except socket.error:
return
if self.verify_request(request, client_address):
@@ -224,6 +229,28 @@ class BaseServer:
self.handle_error(request, client_address)
self.close_request(request)
+ def await_request(self):
+ """Call get_request or handle_timeout, observing self.timeout.
+
+ Returns value from get_request() or raises socket.timeout exception if
+ timeout was exceeded.
+ """
+ if self.timeout is not None:
+ # If timeout == 0, you're responsible for your own fd magic.
+ import select
+ fd_sets = select.select([self], [], [], self.timeout)
+ if not fd_sets[0]:
+ self.handle_timeout()
+ raise socket.timeout("Listening timed out")
+ return self.get_request()
+
+ def handle_timeout(self):
+ """Called if no new request arrives within self.timeout.
+
+ Overridden by ForkingMixIn.
+ """
+ pass
+
def verify_request(self, request, client_address):
"""Verify the request. May be overridden.
@@ -289,6 +316,7 @@ class TCPServer(BaseServer):
- server_bind()
- server_activate()
- get_request() -> request, client_address
+ - handle_timeout()
- verify_request(request, client_address)
- process_request(request, client_address)
- close_request(request)
@@ -301,6 +329,7 @@ class TCPServer(BaseServer):
Class variables that may be overridden by derived classes or
instances:
+ - timeout
- address_family
- socket_type
- request_queue_size (only for stream sockets)
@@ -405,11 +434,12 @@ class ForkingMixIn:
"""Mix-in class to handle each request in a new process."""
+ timeout = 300
active_children = None
max_children = 40
def collect_children(self):
- """Internal routine to wait for died children."""
+ """Internal routine to wait for children that have exited."""
while self.active_children:
if len(self.active_children) < self.max_children:
options = os.WNOHANG
@@ -424,6 +454,13 @@ class ForkingMixIn:
if not pid: break
self.active_children.remove(pid)
+ def handle_timeout(self):
+ """Wait for zombies after self.timeout seconds of inactivity.
+
+ May be extended, do not override.
+ """
+ self.collect_children()
+
def process_request(self, request, client_address):
"""Fork a new subprocess to process the request."""
self.collect_children()
@@ -204,8 +208,9 @@ class BaseServer:
#
# - handle_request() is the top-level call. It calls
- # get_request(), verify_request() and process_request()
- # - get_request() is different for stream or datagram sockets
+ # await_request(), verify_request() and process_request()
+ # - get_request(), called by await_request(), is different for
+ # stream or datagram sockets
# - process_request() is the place that may fork a new process
# or create a new thread to finish the request